package com.yixin.ms.server;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.vo.HealthOpenReturnVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

/**
 * @description:
 * @ClassName HealthOpenReturnServer
 * @Author WXG
 * @Date 2022/10/20 17:34
 */
@Slf4j
@Component
public class HealthOpenReturnServer {

    public HealthOpenReturnVO healthOpenReturn(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO) {
        HealthOpenReturnVO healthOpenReturnVO = new HealthOpenReturnVO();
        /*用户健康码信息*/
        HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
        //用户核酸信息
        List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
        Integer nucleicExpirationTime = placeInfoVO.getNucleicExpirationTime();
        Boolean noXcOpenDoor = placeInfoVO.getNoXcOpenDoor();
        Boolean noHsOpenDoor = placeInfoVO.getNoHsOpenDoor();
        Boolean riskXcOpenDoor = placeInfoVO.getRiskXcOpenDoor();
        String noAllowAreaCode = placeInfoVO.getNoAllowAreaCode();
        Boolean xcNeedCheckOpenDoor = placeInfoVO.getXcNeedCheckOpenDoor();
        Boolean isThroughBlackCheck = placeInfoVO.getIsThroughBlackCheck();
        boolean nucleicExpirationFlag = false;
        Boolean isNoAllowAreaCode = true;//是否是允许通行的身份证地区
        /*
         * 身份证判定校验
         * */
        if (StrUtil.isNotEmpty(noAllowAreaCode) && StrUtil.isNotEmpty(userVO.getIdNumber())) {
            String[] split = noAllowAreaCode.split("/");
            for (String areaCode : split) {
                if (userVO.getIdNumber().startsWith(areaCode)) {
                    isNoAllowAreaCode = false;
                    break;
                }
            }
        }
        String openMsg = "";
        String openVoiceMsg = "";
        /**
         * 开门规则判定校验
         */
        if (placeInfoVO != null && placeInfoVO.getScenicFlag()) {
            userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode());
            openMsg = "绿码，欢迎入园";
            openVoiceMsg = openMsg;
        } else {
            userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode());
            openMsg = "绿码，请通行";
            openVoiceMsg = openMsg;
        }

        //红黄码
        if (!HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = codeTypeVO.getMzt() + "禁止通行";
            openVoiceMsg = openMsg;
            if (HealthCodeLevelEnum.RED.getDesc().equals(codeTypeVO.getMzt())) {
                codeTypeVO.setContent(codeTypeVO.getMzt() + "：健康码状态为高风险").setContentColor(ColorEnum.RED.getValue());
                userVO.setBackgroundColor(ColorEnum.RED.getValue());
            } else {
                codeTypeVO.setContent(codeTypeVO.getMzt() + ": 健康码状态为中风险").setContentColor(ColorEnum.YELLOW.getValue());
                userVO.setBackgroundColor(ColorEnum.YELLOW.getValue());
            }
        }
        //黑名单校验
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !isThroughBlackCheck) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = "未授权，请联系管理员";
            openVoiceMsg = openMsg;
        }
        //不被允许通行的身份证地区
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !isNoAllowAreaCode) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = "禁止通行,需检测";
            openVoiceMsg = openMsg;
        }
        //去过中高风险地区
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !riskXcOpenDoor && StrUtil.isNotEmpty(userVO.getTravelValidation()) && TravelValidationEnum.HAVE_BEEN_TO.getCode().equals(userVO.getTravelValidation())) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = TravelValidationEnum.HAVE_BEEN_TO.getDesc() + ",禁止通行";
            openVoiceMsg = openMsg;
        }
        //核酸校验
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && noHsOpenDoor && nucleicExpirationTime != null && !nucleicExpirationTime.equals(0)) {
            if ((CollectionUtils.isEmpty(nucleinList) || DateUtil.betweenMs(nucleinList.stream().findFirst().get().getShowTime(), DateUtil.date()) > nucleicExpirationTime * 60 * 60 * 1000)) {
                nucleicExpirationFlag = true;
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                if (nucleicExpirationTime <= NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                    openMsg = new StringBuilder().append(nucleicExpirationTime).append("小时内无有效核酸阴性记录，禁止通行").toString();
                } else {
                    openMsg = new StringBuilder().append((nucleicExpirationTime / 24) + 1).append("天内无有效核酸阴性记录，禁止通行").toString();
                }
                openVoiceMsg = openMsg;
            }
        }
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !noHsOpenDoor && nucleicExpirationTime != null && !nucleicExpirationTime.equals(0) && (CollectionUtils.isEmpty(nucleinList) || nucleinList.stream().findFirst().get().getReportTime() == null || DateUtil.betweenMs(nucleinList.stream().findFirst().get().getReportTime(), DateUtil.date()) > nucleicExpirationTime * 60 * 60 * 1000)) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            if (CollectionUtil.isNotEmpty(nucleinList) && DateUtil.betweenMs(nucleinList.stream().findFirst().get().getChecktime(), DateUtil.date()) < 24 * 60 * 60 * 1000) {
                openMsg = "24小时核酸结果未出，禁止通行";
            } else {
                openMsg = nucleicExpirationTime + "小时内无有效核酸阴性记录，禁止通行";
            }
            openVoiceMsg = openMsg;
        }
        //配置了没有行程不开门的账号
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !noXcOpenDoor && (StrUtil.isEmpty(userVO.getTravelValidation()) || TravelValidationEnum.NO_ITINERARY.getCode().equals(userVO.getTravelValidation()))) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = TravelValidationEnum.NO_ITINERARY.getDesc() + ",需人工核验";
            openVoiceMsg = openMsg;
        }
        //行程码需检测不开门
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !xcNeedCheckOpenDoor && StrUtil.isNotEmpty(userVO.getTravelValidation()) && TravelValidationEnum.NEEDCHECK.getCode().equals(userVO.getTravelValidation())) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = "禁止通行,需检测";
            openVoiceMsg = openMsg;
        }
        userVO.setOpenMsg(openMsg);
        userVO.setOpenVoiceMsg(openVoiceMsg);
        /*开门规则校验--结束*/
        healthOpenReturnVO.setNucleicExpirationFlag(nucleicExpirationFlag);
        return healthOpenReturnVO;
    }
}
