package com.yixin.ms.server;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.crypto.util.SzhzSm2Util;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.IDTypeEnum;
import com.yixin.ms.server.dto.DeviceScanCodeDTO;
import com.yixin.ms.server.dto.DeviceScanIdCardDTO;
import com.yixin.ms.server.dto.PlaceRecordDTO;
import com.yixin.ms.server.dto.TokenDataDTO;
import com.yixin.ms.server.vo.DeviceScanIdCardVO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.List;

/**
 * @author: lileilei
 * @date: 2022/6/23 10:47
 * @description: 杭州“四码一屏”统一接口
 */
@Component
@Slf4j
public class PlaceHealthCodeServer {
    private static final String TOKEN_KEY = "health:code:token";
    @Value("${health.code.clientId}")
    private String clientId;

    @Value("${health.code.clientSecret}")
    private String clientSecret;

    @Value("${health.code.publicKey}")
    private String publicKey;

    @Value("${health.code.getTokenUrl}")
    private String getTokenUrl;

    @Value("${health.code.deviceScanIdCardUrl}")
    private String deviceScanIdCardUrl;

    @Value("${health.code.deviceScanIdCardEncryptUrl}")
    private String deviceScanIdCardEncryptUrl;

    @Value("${health.code.deviceScanCodeUrl}")
    private String deviceScanCodeUrl;

    @Value("${health.code.queryPlaceInfoBySnUrl}")
    private String queryPlaceInfoBySnUrl;

    @Value("${health.code.savePlaceRecordUrl}")
    private String savePlaceRecordUrl;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken() {
        HealthResultVO healthResultVO = restTemplate.getForObject(getTokenUrl, HealthResultVO.class);
        log.info("获取token响应内容：{}", JSON.toJSONString(healthResultVO));
        healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
        return healthResultVO;
    }

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken(Boolean cleanSession) {
        HealthResultVO healthResultVO = restTemplate.getForObject(new StringBuilder().append(getTokenUrl).append("?cleanSession=").append(cleanSession).toString(), HealthResultVO.class);
        log.info("获取token响应内容：{}", JSON.toJSONString(healthResultVO));
        healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
        return healthResultVO;
    }


    /**
     * 扫人员证件查询场所码信息
     *
     * @return
     */
    public HealthResultVO<DeviceScanIdCardVO> deviceScanIdCard(DeviceScanIdCardDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken().getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            String secret32Str = SecureUtil.md5(clientSecret).toLowerCase();
            AES aes = SecureUtil.aes(secret32Str.getBytes());
            String idNum = aes.encryptBase64(dto.getIdNumber().getBytes());
            dto.setIdNumber(idNum);
            healthResultVO = restTemplate.postForObject(deviceScanIdCardUrl, dto, HealthResultVO.class);
            log.info("扫人员证件查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("扫人员证件查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }

    public HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(ShortTermDTO shortTermDTO) {
        DeviceScanIdCardDTO scanIdCardDTO = new DeviceScanIdCardDTO().setDeviceId(shortTermDTO.getSn()).setIdNumber(shortTermDTO.getIdNumber());
        if (SMSOrIdCardUtils.isIdCard(shortTermDTO.getIdNumber())) {
            scanIdCardDTO.setIdType(IDTypeEnum.ID_CARD.getCode());
        } else {
            scanIdCardDTO.setIdType(IDTypeEnum.PASSPORT.getCode());
        }

        //扫人员证件查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = this.deviceScanIdCardEncrypt(scanIdCardDTO);
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(RestCode.SUCCESS.getCode())) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(RestCode.INVALID_TOKEN.getCode())) {
                //清理token缓存
                this.getToken(true);
            }
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = this.deviceScanIdCardEncrypt(scanIdCardDTO);
        }
        return scanIdCardResult;
    }

    /**
     * 扫人员证件查询场所码信息(请求体加密)
     *
     * @return
     */
    public HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(DeviceScanIdCardDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken().getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            dto.setIdNumber(dto.getIdNumber());
            String jsonReq = JSON.toJSONString(dto);
            //得到加密的jsonBody,公钥在文档中有，或者找数杭接口人获取
            String requestBody = SzhzSm2Util.encrypt(publicKey, jsonReq);
            //获取最终的传参的JSON串
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("scanIdCardBody", requestBody);
            healthResultVO = restTemplate.postForObject(deviceScanIdCardEncryptUrl, jsonObject, HealthResultVO.class);
            log.info("扫人员证件查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("扫人员证件查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }


    public HealthResultVO<DeviceScanIdCardVO> deviceScanCode(ShortTermDTO shortTermDTO) {
        //健康码查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = this.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(RestCode.SUCCESS.getCode())) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(RestCode.INVALID_TOKEN.getCode())) {
                //清理token缓存
                this.getToken(true);
            }
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = this.deviceScanCode(new DeviceScanCodeDTO().setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode()).setUrlParams(shortTermDTO.getHealthCode()));
        }

        return scanIdCardResult;
    }

    /**
     * 健康码查询场所码信息
     * <p>
     * 根据barcode查询用户健康码信息（四码一屏）
     *
     * @return
     */
    public HealthResultVO<DeviceScanIdCardVO> deviceScanCode(DeviceScanCodeDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken().getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            healthResultVO = restTemplate.postForObject(deviceScanCodeUrl, dto, HealthResultVO.class);
            log.info("健康码查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("健康码查询场所码信息：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }

    /**
     * 通过设备sn查询场所码信息
     */
    public PlaceInfoVO queryPlaceInfoBySn(String sn) {

        PlaceInfoVO placeInfoVO = new PlaceInfoVO();
        if (Preconditions.isNotBlank(sn)) {
            RestResult result = restTemplate.getForObject(new StringBuilder().append(queryPlaceInfoBySnUrl).append("?sn=").append(sn).toString(), RestResult.class);
            placeInfoVO = JSON.parseObject(JSON.toJSONString(result.getData()), PlaceInfoVO.class);
            log.info("通过设备sn: {}, 查询场所码信息返回内容:{}", sn, JSON.toJSONString(result));
            //TODO 暂时没有特定的业务CODE去标识 目前先这样判断
            if (result.getCode() == 500 && "设备编码不存在".equals(result.getMsg())) {
                throw new ServiceException("设备未录入系统，请联系管理员");
            }
        }

        if (Preconditions.isBlank(placeInfoVO)) {
            placeInfoVO = new PlaceInfoVO();
        }

        //特殊逻辑 设备号
        String supremeSn = (String) redisUtil.get("hh_system_config_sn");
        if (supremeSn != null) {
            List<String> sns = Arrays.asList(supremeSn.split("::"));
            //核酸时间 单位小时
            Integer supremeTime = (Integer) redisUtil.get("hh_system_config_nucleicTime");
            if (sns.contains(sn)) {
                placeInfoVO.setNucleicExpirationTime(supremeTime);
            }
        }
        return placeInfoVO;
    }

    /**
     * 保存场所记录
     */
    public RestResult<PlaceInfoVO> savePlaceRecord(PlaceRecordDTO dto) {
        return restTemplate.postForObject(savePlaceRecordUrl, dto, RestResult.class);
    }
}
