package com.yixin.ms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.*;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SignUtils;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ProvinceHealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.*;
import com.yixin.ms.server.dto.*;
import com.yixin.ms.server.vo.*;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;


@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {
    /**
     * 65周岁预约不限制通行
     */
    private static final int AGE = 65;
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private HealthServer healthServer;
    @Autowired
    private ScenicServer scenicServer;
    @Resource
    private Executor executor;
    @Resource
    private UserBlackServer userBlackServer;
    @Resource
    private HealthOpenReturnServer healthOpenReturnServer;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        return healthServer.getShortTermCertificate(shortTermDTO);
    }

    private HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        HealtCodeUserVO userVO = null;
        if (placeInfoVO != null && StrUtil.isNotEmpty(placeInfoVO.getAddressCode())) {
            HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = this.deviceScanIdCardEncrypt(shortTermDTO);
            userVO = this.scanIdCardResultToHealtCodeUserVO(null, scanIdCardResult, shortTermDTO, placeInfoVO);
        }
        if (Preconditions.isBlank(userVO)) {
            userVO = healthServer.getShortTermCertificate(shortTermDTO);
        }
        return userVO;
    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }
    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    public static void main(String[] args) {
        String url = "https://h5.dingtalk.com/healthAct/index.html?stateCouncilBarCode=ffd631d12c5ee80fe77fe0fb4347b4e8&flag=1#/result";
        Map<String, String> map = HttpUtil.decodeParamMap(url, Charset.defaultCharset());

        String barCode = HttpUtil.decodeParamMap(url.toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
    }

    @Override
    public RestResult<HealtCodeUserVO> getShortTerm(ShortTermDTO shortTermDTO) {
        log.info("--->健康码查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(700, "token为空哦");
        }
        /*回参带开门信号*/
        HealtCodeUserVO userVO = null;
        /*换取barCode*/
        if (StrUtil.isNotBlank(shortTermDTO.getHealthCode())) {
            convertHealthCode(shortTermDTO, shortTermDTO.getHealthCode());
        } else {
            if (StringUtils.isBlank(shortTermDTO.getIdNumber()) && StringUtils.isBlank(shortTermDTO.getName())) {
                throw new ServiceException(701, "身份证和姓名为空哦");
            }
        }

        /*查询设备场所码信息   以及设备所绑账号的开关门信息*/
        PlaceInfoVO placeInfoVO = placeHealthCodeServer.queryPlaceInfoBySn(shortTermDTO.getSn());
        if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {
            //扫二维码
            userVO = getHealthCodeInfoByBarcode(shortTermDTO, placeInfoVO);
            if (Preconditions.isBlank(userVO) && Preconditions.isNotBlank(shortTermDTO.getIdNumber())) {
                //barcode兜底逻辑
                userVO = this.getShortTermCertificate(shortTermDTO);
            }
        } else {
            //人脸识别 刷身份证
            if (Preconditions.isNotBlank(shortTermDTO.getIdNumber())) {
                userVO = getShortTermCertificate(shortTermDTO, placeInfoVO);
            }
        }

        /*根据查询到的信息，返回设备显示信息*/
        if (userVO != null) {
            if (CollUtil.isEmpty(userVO.getData())) {
                throw new ServiceException("未查询到健康码");
            }
            /*姓名身份证数据脱敏处理*/
            this.barCodeConvertInfo(userVO, shortTermDTO);
            // 景区场景，需要判断有无预约
            if (Preconditions.isNotBlank(placeInfoVO)
                    && placeInfoVO.getScenicFlag()
                    && Preconditions.isNotBlank(userVO.getIdNumber())
                    && IdcardUtil.getAgeByIdCard(userVO.getIdNumber()) < AGE) {
                //判断是否小于65周岁查询预约放行权限
                scenicServer.verify(new ScenicVerifyDTO().setIdNumber(userVO.getIdNumber()), placeInfoVO.getAdminId());
            }
            /*黑名单校验*/
            Boolean isBlack = userBlackServer.checkUserBlackListURL(userVO.getIdNumber(), shortTermDTO.getSn());
            placeInfoVO.setIsThroughBlackCheck(isBlack);
            /*疫苗 接种数量错误修改  核酸信息修复 健康码信息处理*/
            this.healthVaccineNum(userVO);
            /*回参数据填入*/
            this.resultUserVOInfo(userVO, placeInfoVO, shortTermDTO);
            /*游客数据推送 进入方法先校验是否是景区游客并且通行，才会推送记录*/
            scenicServer.checkConsume(userVO, placeInfoVO, shortTermDTO);
            log.info("开门信号结果返回:{}", JSON.toJSONString(userVO));
            return RestResponse.ok(userVO);
        }

        log.info("未查询到此数据");
        throw new ServiceException(703, "未查询到此数据");
    }

    /*数据回填*/
    private void resultUserVOInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        //用户核酸信息
        List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
        //开关门规则校验 及语音提示
        HealthOpenReturnVO healthOpenReturnVO = healthOpenReturnServer.healthOpenReturn(userVO, placeInfoVO);
        //核酸过期标识
        Boolean nucleicExpirationFlag = healthOpenReturnVO.getNucleicExpirationFlag();
        if (CollectionUtil.isNotEmpty(nucleinList)) {
            HealtCodeNucleinVO nucleinVO = nucleinList.stream().findFirst().get();
            nucleinVO.setTitle("核酸检测").setContentColor(nucleicExpirationFlag ? ColorEnum.YELLOW.getValue() : ColorEnum.GREEN.getValue()).setIconUrl(nucleicExpirationFlag ? NucleicStatusEnum.NOT_AVAILABLE.getUrl() : NucleicStatusEnum.NEGATIVE.getUrl());
            if (nucleinVO.getReportTime() != null && DateUtil.compare(nucleinVO.getChecktime(), nucleinVO.getReportTime()) > 0) {
                nucleinVO.setContent("结果未出");
            } else {
                long betweenHour = DateUtil.between(nucleinVO.getShowTime(), DateUtil.date(), DateUnit.HOUR);
                if (StrUtil.isEmpty(nucleinVO.getResult())) {
                    nucleinVO.setResult("结果未出");
                }
                if (betweenHour < NucleicRuleEnum.TWENTY_FOUR.getValue()) {
                    nucleinVO.setContent(NucleicRuleEnum.TWENTY_FOUR.getValue() + " " + nucleinVO.getResult());
                } else if (betweenHour < NucleicRuleEnum.FORTY_EIGHT.getValue()) {
                    nucleinVO.setContent(NucleicRuleEnum.FORTY_EIGHT.getValue() + " " + nucleinVO.getResult());
                } else if (betweenHour < NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                    nucleinVO.setContent(NucleicRuleEnum.SEVENTY_TWO.getValue() + " " + nucleinVO.getResult());
                } else {
                    nucleinVO.setContent((DateUtil.betweenDay(nucleinVO.getShowTime(), DateUtil.date(), false) + 1) + "天 " + nucleinVO.getResult());
                }
            }
        } else {
            userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setTitle("核酸检测").setIconUrl(NucleicStatusEnum.NOT_AVAILABLE.getUrl()).setContent("暂无").setContentColor(ColorEnum.GREEN.getValue())));
        }

        userVO.setTitle("健 康 码").setTitleColor(ColorEnum.WHITE.getValue())
                //疫苗信息
                .setVaccineIcoTitle("新冠疫苗").setVaccineIconUrl(HealthVaccineNumEnum.numOf(userVO.getVaccineNum()).getUrl()).setVaccineNumColor(ColorEnum.GREEN.getValue())
                //行程信息
                .setTravelValidationDesc(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getDesc()).setTravelValidationTitle("动态行程卡").setTravelValidationIconUrl(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getUrl());
    }

    private HealtCodeUserVO getHealthCodeInfoByBarcode(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        HealtCodeUserVO userVO = null;
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = null;
        if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
            if (placeInfoVO != null) {
                /*绑定场所码*/
//                if (StrUtil.isEmpty(placeInfoVO.getAddressCode())) {
//                    throw new ServiceException("设备扫码未绑定场所码，请联系管理员");
//                }
            } else {
                throw new ServiceException(703, "未查询到此数据");
            }

            if (NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
                //TODO 以后可能就要走策略模式 根据SN获取adminId配置的策略 调用不同的获取用户身份信息的接口
                //预约码查询
                ScenicVerifyVO verifyVO = scenicServer.verify(new ScenicVerifyDTO().setReservationCode(shortTermDTO.getHealthCode()), placeInfoVO.getAdminId());
                shortTermDTO.setName(verifyVO.getUserName()).setIdNumber(verifyVO.getIdNumber());
                scanIdCardResult = this.deviceScanIdCardEncrypt(shortTermDTO);

            } else if (!NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
                //健康码查询（四码一屏） 是不返回姓名和身份证号的
                CompletableFuture<HealthResultVO<DeviceScanIdCardVO>> healthResultVOCompletableFuture = CompletableFuture.supplyAsync(() -> deviceScanCode(shortTermDTO));
                //barcode换取用户身份信息 身份证和姓名 未脱敏
                CompletableFuture<HealthCodeVO> HealthCodeVOCompletableFuture = CompletableFuture.supplyAsync(() -> healthServer.queryInfoByBarCode(shortTermDTO.getHealthCode()));
                try {
                    scanIdCardResult = healthResultVOCompletableFuture.get();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                try {
                    HealthCodeVO healthCodeVO = HealthCodeVOCompletableFuture.get();
                    if (Preconditions.isNotBlank(healthCodeVO)) {
                        shortTermDTO.setName(healthCodeVO.getUserName()).setIdNumber(healthCodeVO.getIdNum());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            userVO = this.scanIdCardResultToHealtCodeUserVO(userVO, scanIdCardResult, shortTermDTO, placeInfoVO);
        } else {
            throw new ServiceException("请扫描正确的二维码！");
        }
        return userVO;
    }

    private HealtCodeUserVO scanIdCardResultToHealtCodeUserVO(HealtCodeUserVO userVO, HealthResultVO<DeviceScanIdCardVO> scanIdCardResult, ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        if (scanIdCardResult != null && scanIdCardResult.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
            DeviceScanIdCardVO scanIdCardVO = scanIdCardResult.getData();
            /*数据转换*/
            userVO = this.deviceScanIdCardVOConvert(scanIdCardVO, shortTermDTO);
            /*场所码数据推送*/
            if (Preconditions.isNotBlank(placeInfoVO)
                    && Preconditions.isNotBlank(placeInfoVO.getAddressCode())
                    && Preconditions.isNotBlank(placeInfoVO.getManagerHealthCode())) {
                this.savePlaceRecord(shortTermDTO, scanIdCardVO, placeInfoVO);
            }
        }
        return userVO;
    }


    private HealtCodeUserVO deviceScanIdCardVOConvert(DeviceScanIdCardVO scanIdCardVO, ShortTermDTO shortTermDTO) {
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        //疫苗接种数量
        userVO.setVaccineNum(scanIdCardVO.getVaccinated());
        //核酸记录
        if (StrUtil.isNotEmpty(scanIdCardVO.getCheckTime())) {
            userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setChecktime(DateUtil.parseDateTime(scanIdCardVO.getCheckTime())).setReportTime(StrUtil.isEmpty(scanIdCardVO.getReportTime()) ? null : DateUtil.parseDateTime(scanIdCardVO.getReportTime())).setResult(scanIdCardVO.getCheckResult())));
        } else {
            userVO.setNuclein(Lists.newArrayList());
        }
        //健康码信息
        userVO.setData(Lists.newArrayList(new HealtCodeTypeVO().setMzt(HealthCodeLevelEnum.codeOf(scanIdCardVO.getLevel()).getDesc())));
        //行程信息
        userVO.setTravelValidation(scanIdCardVO.getTravelValidation());
        userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? scanIdCardVO.getUsername() : shortTermDTO.getName());
        return userVO;
    }

    private void savePlaceRecord(ShortTermDTO shortTermDTO, DeviceScanIdCardVO scanIdCardVO, PlaceInfoVO placeInfoVO) {
        PlaceRecordDTO placeRecordDTO = new PlaceRecordDTO().setName(StrUtil.isEmpty(shortTermDTO.getName()) ? scanIdCardVO.getUsername() : shortTermDTO.getName()).setIdNumber(shortTermDTO.getIdNumber()).setLevel(HealthCodeLevelEnum.codeOf(scanIdCardVO.getLevel()).getDesc())
                //检测时间
                .setCheckTime(scanIdCardVO.getCheckTime() == null ? null : DateUtil.parseDateTime(scanIdCardVO.getCheckTime()))
                //报告时间
                .setReportTime(scanIdCardVO.getReportTime() == null ? null : DateUtil.parseDateTime(scanIdCardVO.getReportTime()))
                //检测结果
                .setCheckResult(scanIdCardVO.getCheckResult())
                .setTravelValidation(scanIdCardVO.getTravelValidation())
                .setVaccinated(scanIdCardVO.getVaccinated())
                .setAddressCode(placeInfoVO.getAddressCode())
                .setUnitName(placeInfoVO.getUnitName())
                .setUnitAddress(placeInfoVO.getUnitAddress())
                .setManagerHealthCode(placeInfoVO.getManagerHealthCode())
                .setAdminId(placeInfoVO.getAdminId())
                .setHealthCode(shortTermDTO.getHealthCode())
                .setCreateTime(DateUtil.date());
        executor.execute(() -> {
            //保存场所记录
            placeHealthCodeServer.savePlaceRecord(placeRecordDTO);
        });
    }

    /**
     * 处理并提取healthCode
     *
     * @param healthCodeSource 000026https://h5.dingtalk.com/healthAct/index.html?stateCouncilBarCode=ffd631d12c5ee80fe77fe0fb4347b4e8&flag=1#/result
     *                         扫码参数兼容问题处理
     */
    private void convertHealthCode(ShortTermDTO shortTermDTO, String healthCodeSource) {

        if (!shortTermDTO.getHealthCode().contains("http")
                && !shortTermDTO.getHealthCode().contains("HTTP") && !NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
            throw new ServiceException("请打开支付宝选择杭州健康码重试");
        }

        String barCode = null;
        //获取url中的barCode信息
        if (HttpUtil.isHttp(shortTermDTO.getHealthCode()) || HttpUtil.isHttps(shortTermDTO.getHealthCode())) {
            if (shortTermDTO.getHealthCode().contains("http")) {
                barCode = HttpUtil.decodeParamMap(healthCodeSource.substring(healthCodeSource.indexOf("http")).toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
            }
            if (shortTermDTO.getHealthCode().contains("HTTP")) {
                barCode = HttpUtil.decodeParamMap(healthCodeSource.substring(healthCodeSource.indexOf("HTTP")).toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
            }
            shortTermDTO.setHealthCode(barCode);
        }

        if (StrUtil.isBlank(shortTermDTO.getHealthCode())) {
            throw new ServiceException("请打开支付宝选择杭州健康码重试");
        }
    }

    /**
     * 根据barCode查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanCode(ShortTermDTO shortTermDTO) {
        return placeHealthCodeServer.deviceScanCode(shortTermDTO);

    }

    /**
     * 根据证件号查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(ShortTermDTO shortTermDTO) {
        return placeHealthCodeServer.deviceScanIdCardEncrypt(shortTermDTO);
    }

    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }
        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }

    /*在有barCode情况下获取个人信息回填*/
    private void barCodeConvertInfo(HealtCodeUserVO userVO, ShortTermDTO shortTermDTO) {
        //姓名和身份证号脱敏处理
        //姓名 通过barcode获取真实姓名
        if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())
                && !NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
            HealthCodeVO healthCodeVO = healthServer.queryInfoByBarCode(shortTermDTO.getHealthCode());
            if (healthCodeVO != null) {
                if (healthCodeVO.getUserName() != null) {
                    userVO.setUsername(healthCodeVO.getUserName());
                }
                if (StrUtil.isNotEmpty(healthCodeVO.getPn())) {
                    userVO.setPhone(healthCodeVO.getPn());
                }

                if (healthCodeVO.getIdNum() != null) {
                    userVO.setIdNumber(healthCodeVO.getIdNum());
                    userVO.setShowIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
                } else {
                    //如果返回为null那就设置barcode给idNumber
                    userVO.setIdNumber(shortTermDTO.getHealthCode());
                    userVO.setShowIdNumber(StrUtil.EMPTY);
                }
            } else {
                //如果返回为null那就设置barcode给idNumber
                userVO.setIdNumber(shortTermDTO.getHealthCode());
                userVO.setShowIdNumber(StrUtil.EMPTY);
            }

        }
        userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
        userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setShowUsername(StrUtil.isEmpty(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
    }

    /*
    1.疫苗接种数量修缮
    2.核酸信息处理
    3.健康码信息处理
    * */
    private void healthVaccineNum(HealtCodeUserVO userVO) {
        /**
         * 因为返回参数和针数不是一一对应所以这里做处理，修正
         * 0 未接种 对应0针
         * 1-接种完成 对应2针
         * 2-未完全接种 对应1针
         * 3- 完成加强针 对应3针
         */
        HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
        if (HealthVaccineNumEnum.VACCINATION_COMPLETE.getStatus().equals(userVO.getVaccineNum())) {
            userVO.setVaccineNum(HealthVaccineNumEnum.VACCINATION_COMPLETE.getNum());
        } else if (HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getStatus().equals(userVO.getVaccineNum())) {
            userVO.setVaccineNum(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getNum());
        } else {
            userVO.setVaccineNum(userVO.getVaccineNum());
        }
        /*核酸信息处理*/
        List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
        if (CollectionUtil.isNotEmpty(nucleinList)) {
            nucleinList.forEach(v -> {
                Date date = v.getReportTime() == null ? v.getChecktime() : DateUtil.compare(v.getChecktime(), v.getReportTime()) > 0 ? v.getChecktime() : v.getReportTime();
                v.setShowTime(date);
                v.setChecktime(v.getChecktime());
                v.setReportTime(v.getReportTime());
            });
        }
        /*健康码处理*/
        if (HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
            codeTypeVO.setContent("绿码：健康码状态为低风险").setContentColor(ColorEnum.GREEN.getValue());
            userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
        }
    }

    @Override
    public RestResult<HealtCodeUserVO> queryProvinceHealthCode(ProvinceHealthCodeDTO healthCodeDTO) {

        //校验参数
        if (Preconditions.isBlank(healthCodeDTO.getIdNumber())
                || Preconditions.isBlank(healthCodeDTO.getAppKey())
                || Preconditions.isBlank(healthCodeDTO.getSign())
                || Preconditions.isBlank(healthCodeDTO.getReqTimestamp())) {
            throw new ServiceException(ErrorCodeEnum.PARAMETER_ERROR.getErrorCode(), ErrorCodeEnum.PARAMETER_ERROR.getErrorMsg());
        }

        //校验签名
        if (!SignUtils.checkSign(IrsTripAuthEnum.PROVINCE_HEALTH_CODE_OPEN_API.getAppKey(),
                IrsTripAuthEnum.PROVINCE_HEALTH_CODE_OPEN_API.getAppSecret(),
                healthCodeDTO.getSign(), healthCodeDTO.getReqTimestamp())) {
            throw new ServiceException(ErrorCodeEnum.SIGN_ERROR.getErrorCode(), ErrorCodeEnum.SIGN_ERROR.getErrorMsg());
        }

        //增加上层接口 省健康码基本信息、疫苗注射数据信息、核酸检测数据 自定义封装三合一接口
        HealtCodeUserVO userVO = healthServer.getProvinceHealthCodeInfo(healthCodeDTO.getIdNumber());
        log.info("---->省健康码三合一接口：{}", JSON.toJSONString(userVO));
        if (userVO != null && !CollectionUtils.isEmpty(userVO.getData())) {
            convertProvinceHealthCodeInfo(userVO);
        }

        return RestResponse.ok(userVO);
    }

    private void convertProvinceHealthCodeInfo(HealtCodeUserVO userVO) {

        if (userVO != null) {
            if (Preconditions.isBlank(userVO)
                    || Preconditions.isBlank(userVO.getData())) {
                throw new ServiceException("未查询到健康码");
            }

            //姓名和身份证号脱敏处理
            userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? StrUtil.EMPTY : userVO.getUsername());
            userVO.setShowUsername(StrUtil.isEmpty(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
            userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : userVO.getIdNumber());
            userVO.setShowIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));

            /**
             * 0 未接种 对应0针
             *
             * 1-接种完成 对应2针
             *
             * 2-未完全接种 对应1针
             *
             * 3- 完成加强针 对应3针
             */
            if (HealthVaccineNumEnum.VACCINATION_COMPLETE.getStatus().equals(userVO.getVaccineNum())) {
                userVO.setVaccineNum(HealthVaccineNumEnum.VACCINATION_COMPLETE.getNum());
            } else if (HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getStatus().equals(userVO.getVaccineNum())) {
                userVO.setVaccineNum(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getNum());
            } else {
                userVO.setVaccineNum(userVO.getVaccineNum());
            }

            //用户健康码信息
            HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
            //用户核酸信息
            List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
            if (CollectionUtil.isNotEmpty(nucleinList)) {
                nucleinList.forEach(v -> {
                    Date date = v.getReportTime() == null ? v.getChecktime() : DateUtil.compare(v.getChecktime(), v.getReportTime()) > 0 ? v.getChecktime() : v.getReportTime();
                    v.setShowTime(date);
                    v.setChecktime(v.getChecktime());
                    v.setReportTime(v.getReportTime());
                });
            }

            if (HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
                codeTypeVO.setContent("绿码：健康码状态为低风险").setContentColor(ColorEnum.GREEN.getValue());
                userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
            }
            userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode()).setOpenMsg("绿码，请通行").setOpenVoiceMsg("绿码，请通行");


            //核酸过期标识
            boolean nucleicExpirationFlag = false;

            /**
             * 开门规则判定校验：红黄码，核酸有效期
             */
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                String openMsg = new StringBuilder().append(codeTypeVO.getMzt()).append("禁止通行").toString();
                userVO.setOpenMsg(openMsg).setOpenVoiceMsg(openMsg);
                if (HealthCodeLevelEnum.RED.getDesc().equals(codeTypeVO.getMzt())) {
                    codeTypeVO.setContent(new StringBuilder().append(codeTypeVO.getMzt()).append("：健康码状态为高风险").toString()).setContentColor(ColorEnum.RED.getValue());
                    userVO.setBackgroundColor(ColorEnum.RED.getValue());
                } else {
                    codeTypeVO.setContent(new StringBuilder().append(codeTypeVO.getMzt()).append(": 健康码状态为中风险").toString()).setContentColor(ColorEnum.YELLOW.getValue());
                    userVO.setBackgroundColor(ColorEnum.YELLOW.getValue());
                }
            }

            //去过中高风险地区
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && StrUtil.isNotEmpty(userVO.getTravelValidation()) && TravelValidationEnum.HAVE_BEEN_TO.getCode().equals(userVO.getTravelValidation())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                userVO.setOpenMsg(TravelValidationEnum.HAVE_BEEN_TO.getDesc() + ",禁止通行");
                userVO.setOpenVoiceMsg(TravelValidationEnum.HAVE_BEEN_TO.getDesc() + ",禁止通行");
            }

            if (CollectionUtil.isNotEmpty(nucleinList)) {
                HealtCodeNucleinVO nucleinVO = nucleinList.stream().findFirst().get();
                nucleinVO.setTitle("核酸检测").setContentColor(nucleicExpirationFlag ? ColorEnum.YELLOW.getValue() : ColorEnum.GREEN.getValue()).setIconUrl(nucleicExpirationFlag ? NucleicStatusEnum.NOT_AVAILABLE.getUrl() : NucleicStatusEnum.NEGATIVE.getUrl());
                if (nucleinVO.getReportTime() != null && DateUtil.compare(nucleinVO.getChecktime(), nucleinVO.getReportTime()) > 0) {
                    nucleinVO.setContent(new StringBuilder().append("结果未出").toString());
                } else {
                    long betweenHour = DateUtil.between(nucleinVO.getShowTime(), DateUtil.date(), DateUnit.HOUR);
                    if (StrUtil.isEmpty(nucleinVO.getResult())) {
                        nucleinVO.setResult("结果未出");
                    }
                    if (betweenHour < NucleicRuleEnum.TWENTY_FOUR.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.TWENTY_FOUR.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else if (betweenHour < NucleicRuleEnum.FORTY_EIGHT.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.FORTY_EIGHT.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else if (betweenHour < NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.SEVENTY_TWO.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else {
                        nucleinVO.setContent(new StringBuilder().append(DateUtil.betweenDay(nucleinVO.getShowTime(), DateUtil.date(), false) + 1).append("天 ").append(nucleinVO.getResult()).toString());
                    }
                }
            } else {
                userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setTitle("核酸检测").setIconUrl(NucleicStatusEnum.NOT_AVAILABLE.getUrl()).setContent("暂无").setContentColor(ColorEnum.GREEN.getValue())));
            }

            userVO.setTitle("健 康 码").setTitleColor(ColorEnum.WHITE.getValue())
                    //疫苗信息
                    .setVaccineIcoTitle("新冠疫苗").setVaccineIconUrl(HealthVaccineNumEnum.numOf(userVO.getVaccineNum()).getUrl()).setVaccineNumColor(ColorEnum.GREEN.getValue())
                    //行程信息
                    .setTravelValidationDesc(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getDesc()).setTravelValidationTitle("动态行程卡").setTravelValidationIconUrl(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getUrl());

            log.info("最后结果返回:{}", JSON.toJSONString(userVO));
        }
    }
}

